// Copyright (c) 2001, Matt Saettler. All Rights Reserved.

#include "duke3d.h"
#include "concom.h"
#include "cryptio.h"

//  Encrypted IO routines

static char *g_pEncryptKey=NULL;
static int g_nEncryptKeySize=0;
static int g_nCurKeyOffset=0;

char EncryptKeyValid()
{
	return (g_nEncryptKeySize!=0);
}

size_t ReadSupportMagic(long lMagic, char *pBuffer, size_t numBytes)
{
	FILE *fp;
	char idfile[256];
	size_t nBytesRead;

	fp = (FILE *)fopen("cdrom.ini","rt");
	if(fp == (FILE *) NULL)
	{
		return 0;
	}

	fscanf(fp,"%s",idfile);
	fclose(fp);

	strcat(idfile,IDFILENAME);

	if( testcd(idfile) )
	{
		return 0;
	}
	fp=fopen(idfile,"RB");
	if(fp == NULL)
	{
		return 0;
	}
	fseek(fp,lMagic,SEEK_SET);
	nBytesRead=fread(pBuffer,1,numBytes,fp);
	fclose(fp);
	return nBytesRead;
}

long SupportCrypt(char *pBuffer, long lBytes)
{
	long lCurByte;
	
	if(g_nCurKeyOffset >= g_nEncryptKeySize)
		g_nCurKeyOffset=0;

	if(g_nCurKeyOffset >= g_nEncryptKeySize)
		return 0;	// must have been zero.. return no bytes encrypted

	// for the entire buffer, use the encrypt key as XOR.  Repeat key
	// as needed.
	for(lCurByte=0;lCurByte<lBytes;lCurByte++)
	{
		pBuffer[lCurByte]=pBuffer[lCurByte]^g_pEncryptKey[g_nCurKeyOffset];
		g_pEncryptKey[g_nCurKeyOffset]++;	// increment key so it doesn't repeat.
		g_nCurKeyOffset++;	// next byte
		if(g_nCurKeyOffset >=g_nEncryptKeySize)
		{
			g_nCurKeyOffset=0;
		}
	}
	return lBytes;
}

CryptSetKey(long lMagic, int EncryptKeySize)
{
	char *pEncryptKey;
	
	if(g_pEncryptKey)
	{
		free(g_pEncryptKey);
	}
	g_pEncryptKey=(char *)malloc(EncryptKeySize);
	
	g_nEncryptKeySize=ReadSupportMagic(lMagic,g_pEncryptKey,EncryptKeySize);
	g_nCurKeyOffset=0;
}

CryptFRead(void *buffer, size_t dasizeof, size_t count, long fil)
{
	long leng;
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	
	// use kfread to allow reading from GRP file.
	leng=kread(fil,buffer,dasizeof*count);	// read the raw data
	
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);

	SupportCrypt(buffer,leng);	// decrypt
	
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	return leng;
}

CryptFWrite(void *buffer, size_t dasizeof, size_t count, FILE *fil)
{
	long leng;
	char *pEncrypt;

//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	pEncrypt=malloc(dasizeof*count);
	if(!pEncrypt)
		return 0;

//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	memcpy(pEncrypt,buffer, dasizeof*count);	// target, source
	
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	SupportCrypt(pEncrypt,dasizeof*count);

//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	
	leng=fwrite(pEncrypt,dasizeof,count,fil);
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);

	free(pEncrypt);
	
//sprintf(g_szBuf,"CP:%s %d",__FILE__,__LINE__);
//AddLog(g_szBuf);
	return leng;
	
}

